{-# LANGUAGE TypeApplications #-}
module Control.Addressbook.Query where

import Data.Text
       (Text)
import qualified Data.Text as T
import qualified Data.Text.Encoding as TE

import Conduit
import qualified Data.Conduit.Binary as CB
import qualified Data.Conduit.Combinators as C
import qualified Data.Conduit.List as CL
import qualified Data.Conduit.Text as CT

import qualified Data.Map.Strict as Map

import System.IO
       (stdout)

import Data.Maybe
       (fromMaybe)
import System.Environment
       (lookupEnv)
import System.FilePath
       ((</>))

import Control.Exception
       (catch)
import Data.ByteString (ByteString)
import qualified Data.ByteString.Char8 as B

-- XXX: The current Data.Trie implementation is much slower than Data.Set
query :: Text -> IO ()
query prefix = do
  datDir <- fromMaybe "./" <$> lookupEnv "HOME"
  state <- catch @IOError (runResourceT $ runConduit $ readState datDir) (\_ -> pure Map.empty)
  runConduit $ outputResults state
  where
    readState :: FilePath -> ConduitM () Void (ResourceT IO) (Map.Map ByteString [Text])
    readState dir =
      CB.sourceFile (dir </> ".addressbook.dat")
        .| CT.decode CT.utf8
        .| CT.lines
        .| C.foldMap (\s -> Map.singleton (TE.encodeUtf8 $ T.toLower s) [s])
    prefixB = TE.encodeUtf8 prefix
    outputResults :: Map.Map ByteString [Text] -> ConduitM () Void IO ()
    outputResults state =
      CL.sourceList (Map.elems $ Map.filterWithKey (\k _ -> prefixB `B.isPrefixOf` k)  state)
        .| C.concat
        .| C.map (<> "\n")
        .| CT.encode CT.utf8
        .| CB.sinkHandle stdout
